@@ -45,18 +45,18 @@ void usage(const char *const name)
 {
 	if (iamrestorecon) {
 		fprintf(stderr,
-			"usage:  %s [-iFnprRv0] [-e excludedir] [-o filename] pathname...\n"
-			"usage:  %s [-iFnprRv0] [-e excludedir] [-o filename] -f filename\n",
+			"usage:  %s [-iFnprRv0] [-e excludedir] pathname...\n"
+			"usage:  %s [-iFnprRv0] [-e excludedir] -f filename\n",
 			name, name);
 	} else {
 		fprintf(stderr,
-			"usage:  %s [-dilnpqvFW] [-e excludedir] [-o filename] [-r alt_root_path] spec_file pathname...\n"
-			"usage:  %s [-dilnpqvFW] [-e excludedir] [-o filename] [-r alt_root_path] spec_file -f filename\n"
-			"usage:  %s -s [-dilnpqvFW] [-o filename] spec_file\n"
+			"usage:  %s [-dilnpqvFW] [-e excludedir] [-r alt_root_path] spec_file pathname...\n"
+			"usage:  %s [-dilnpqvFW] [-e excludedir] [-r alt_root_path] spec_file -f filename\n"
+			"usage:  %s -s [-dilnpqvFW] spec_file\n"
 			"usage:  %s -c policyfile spec_file\n",
 			name, name, name, name);
 	}
-	exit(1);
+	exit(-1);
 }
 
 static int nerr = 0;
@@ -66,7 +66,7 @@ void inc_err()
 	nerr++;
 	if (nerr > ABORT_ON_ERRORS - 1 && !r_opts.debug) {
 		fprintf(stderr, "Exiting after %d errors.\n", ABORT_ON_ERRORS);
-		exit(1);
+		exit(-1);
 	}
 }
 
@@ -80,7 +80,7 @@ void set_rootpath(const char *arg)
 	if (NULL == r_opts.rootpath) {
 		fprintf(stderr, "%s:  insufficient memory for r_opts.rootpath\n",
 			r_opts.progname);
-		exit(1);
+		exit(-1);
 	}
 
 	/* trim trailing /, if present */
@@ -98,7 +98,7 @@ int canoncon(char **contextp)
 	if (policyfile) {
 		if (sepol_check_context(context) < 0) {
 			fprintf(stderr, "invalid context %s\n", context);
-			exit(1);
+			exit(-1);
 		}
 	} else if (security_canonicalize_context_raw(context, &tmpcon) == 0) {
 		free(context);
@@ -159,6 +159,7 @@ int main(int argc, char **argv)
 	/* Initialize variables */
 	r_opts.progress = 0;
 	r_opts.count = 0;
+	r_opts.nfile = 0;
 	r_opts.debug = 0;
 	r_opts.change = 1;
 	r_opts.verbose = 0;
@@ -174,7 +175,7 @@ int main(int argc, char **argv)
 	r_opts.progname = strdup(argv[0]);
 	if (!r_opts.progname) {
 		fprintf(stderr, "%s:  Out of memory!\n", argv[0]);
-		exit(1);
+		exit(-1);
 	}
 	base = basename(r_opts.progname);
 	
@@ -222,7 +223,7 @@ int main(int argc, char **argv)
 	}
 
 	/* This must happen before getopt. */
-	exclude_non_seclabel_mounts();
+	r_opts.nfile = exclude_non_seclabel_mounts();
 
 	/* Process any options. */
 	while ((opt = getopt(argc, argv, "c:de:f:hilno:pqrsvFRW0")) > 0) {
@@ -241,7 +242,7 @@ int main(int argc, char **argv)
 					fprintf(stderr,
 						"Error opening %s: %s\n",
 						policyfile, strerror(errno));
-					exit(1);
+					exit(-1);
 				}
 				__fsetlocking(policystream,
 					      FSETLOCKING_BYCALLER);
@@ -251,7 +252,7 @@ int main(int argc, char **argv)
 					fprintf(stderr,
 						"Error reading policy %s: %s\n",
 						policyfile, strerror(errno));
-					exit(1);
+					exit(-1);
 				}
 				fclose(policystream);
 
@@ -267,7 +268,7 @@ int main(int argc, char **argv)
 				break;
 			}
 			if (add_exclude(optarg))
-				exit(1);
+				exit(-1);
 			break;
 		case 'f':
 			use_input_file = 1;
@@ -317,13 +318,13 @@ int main(int argc, char **argv)
 			if (optind + 1 >= argc) {
 				fprintf(stderr, "usage:  %s -r rootpath\n",
 					argv[0]);
-				exit(1);
+				exit(-1);
 			}
 			if (NULL != r_opts.rootpath) {
 				fprintf(stderr,
 					"%s: only one -r can be specified\n",
 					argv[0]);
-				exit(1);
+				exit(-1);
 			}
 			set_rootpath(argv[optind++]);
 			break;
@@ -336,7 +337,7 @@ int main(int argc, char **argv)
 			if (r_opts.progress) {
 				fprintf(stderr,
 					"Progress and Verbose mutually exclusive\n");
-				exit(1);
+				exit(-1);
 			}
 			r_opts.verbose++;
 			break;
@@ -346,7 +347,7 @@ int main(int argc, char **argv)
 					"Progress and Verbose mutually exclusive\n");
 				usage(argv[0]);
 			}
-			r_opts.progress = 1;
+			r_opts.progress++;
 			break;
 		case 'W':
 			warn_no_match = 1;
@@ -360,6 +361,14 @@ int main(int argc, char **argv)
 		}
 	}
 
+	for (i = optind; i < argc; i++) {
+		if (!strcmp(argv[i], "/")) {
+			mass_relabel = 1;
+			if (r_opts.progress)
+				r_opts.progress++;
+		}
+	}
+
 	if (!iamrestorecon) {
 		if (policyfile) {
 			if (optind != (argc - 1))
@@ -382,12 +391,12 @@ int main(int argc, char **argv)
 
 		if (stat(argv[optind], &sb) < 0) {
 			perror(argv[optind]);
-			exit(1);
+			exit(-1);
 		}
 		if (!S_ISREG(sb.st_mode)) {
 			fprintf(stderr, "%s:  spec file %s is not a regular file.\n",
 				argv[0], argv[optind]);
-			exit(1);
+			exit(-1);
 		}
 
 		altpath = argv[optind];
@@ -400,7 +409,7 @@ int main(int argc, char **argv)
 	r_opts.selabel_opt_path = altpath;
 
 	if (nerr)
-		exit(1);
+		exit(-1);
 
 	restore_init(&r_opts);
 	if (use_input_file) {
@@ -426,12 +435,8 @@ int main(int argc, char **argv)
 		if (strcmp(input_filename, "-") != 0)
 			fclose(f);
 	} else {
-		for (i = optind; i < argc; i++) {
-			if (!strcmp(argv[i], "/"))
-				mass_relabel = 1;
-
+		for (i = optind; i < argc; i++)
 			errors |= process_glob(argv[i], recurse) < 0;
-		}
 	}
 	
 	maybe_audit_mass_relabel(mass_relabel, errors);
@@ -447,5 +452,5 @@ int main(int argc, char **argv)
 
 	if (r_opts.progress && r_opts.count >= STAR_COUNT)
 		printf("\n");
-	exit(errors);
+	exit(errors ? -1: 0);
 }
